from subprocess import CalledProcessError
import tempfile
import unittest

from helpers import (
    command,
    DictDiffer,
    load_pickle,
    save_pickle,
    unit_info,
    )

from subprocess import CalledProcessError


class TestCommand(unittest.TestCase):

    def testSimpleCommand(self):
        # Creating a simple command (ls) works and running the command
        # produces a string.
        ls = command('/bin/ls')
        self.assertIsInstance(ls(), str)

    def testArguments(self):
        # Arguments can be passed to commands.
        ls = command('/bin/ls')
        self.assertIn('Usage:', ls('--help'))

    def testMissingExecutable(self):
        # If the command does not exist, an OSError (No such file or
        # directory) is raised.
        bad = command('this command does not exist')
        with self.assertRaises(OSError) as info:
            bad()
        self.assertEqual(2, info.exception.errno)

    def testError(self):
        # If the command returns a non-zero exit code, an exception is raised.
        ls = command('/bin/ls')
        with self.assertRaises(CalledProcessError):
            ls('--not a valid switch')

    def testBakedInArguments(self):
        # Arguments can be passed when creating the command as well as when
        # executing it.
        ll = command('/bin/ls', '-l')
        self.assertIn('rw', ll()) # Assumes a file is r/w in the pwd.
        self.assertIn('Usage:', ll('--help'))

    def testQuoting(self):
        # There is no need to quote special shell characters in commands.
        ls = command('/bin/ls')
        ls('--help', '>')

    def testOneLongArgument(self):
        ls = command('/bin/ls')
        with self.assertRaises(CalledProcessError):
            ls('tests.py install')

    def testSplitStringWorks(self):
        ls = command('/bin/ls')
        ls(*'tests.py install'.split())


class TestDictDiffer(unittest.TestCase):

    def testStr(self):
        a = dict(cow='moo', pig='oink')
        b = dict(cow='moo', pig='oinkoink', horse='nay')
        diff = DictDiffer(b, a)
        s = str(diff)
        self.assertIn("added: {'horse': None} -> {'horse': 'nay'}", s)
        self.assertIn("removed: {} -> {}", s)
        self.assertIn("changed: {'pig': 'oink'} -> {'pig': 'oinkoink'}", s)
        self.assertIn("unchanged: ['cow']", s)

    def testStrUnmodified(self):
        a = dict(cow='moo', pig='oink')
        diff = DictDiffer(a, a)
        s = str(diff)
        self.assertEquals('no changes', s)

    def testAddedOrChanged(self):
        a = dict(cow='moo', pig='oink')
        b = dict(cow='moo', pig='oinkoink', horse='nay')
        diff = DictDiffer(b, a)
        expected = set(['horse', 'pig'])
        self.assertEquals(expected, diff.added_or_changed)


class TestPicklers(unittest.TestCase):

    def testSaveAndLoad(self):
        fd = tempfile.NamedTemporaryFile()
        fn = fd.name
        fd.close()
        orig = dict(a=1, b=2)
        save_pickle(orig, fn)
        retrieved = load_pickle(fn)
        self.assertEquals(orig, retrieved)


class TestUnit_info(unittest.TestCase):

    def make_data(self, state='started'):
        return {
            'machines': {0: {
                'dns-name': 'localhost',
                'instance-id': 'local',
                'instance-state': 'running',
                'state': 'running'}},
            'services': {'foo-service': {
                'charm': 'local:oneiric/foo-service-77',
                'relations': {},
                'units': {'foo-unit/29': {
                    'machine': 0,
                    'public-address': '192.168.122.160',
                    'relations': {},
                    'state': state}}}}}

    def testDataParameter(self):
        # The unit_info function can take a data parameter (primarily for
        # testing) that provides the juju service information to be queried.
        # If not provided the "juju status" command is run and its results
        # parsed.
        unit_info('foo-service', 'state', data=self.make_data())

    def testStateFetching(self):
        # The most common data to fetch about a unit is its state.
        state = unit_info('foo-service', 'state', data=self.make_data())
        self.assertEqual('started', state)

    def testFailedState(self):
        state = unit_info(
            'foo-service', 'state', data=self.make_data(state='bad'))
        self.assertNotEqual('started', state)


if __name__ == '__main__':
    unittest.main()
