# Copyright 2012 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

"""Helper functions for writing hooks in python."""

__metaclass__ = type
__all__ = [
    'command',
    'DictDiffer',
    'get_config',
    'install_extra_repository',
    'load_pickle',
    'log',
    'run',
    'save_pickle',
    'unit_info',
    ]

import json
import os
import pickle
import subprocess
import sys
from textwrap import dedent
import yaml


def command(*base_args):
    def callable_command(*args):
        all_args = base_args + args
        return subprocess.check_output(all_args, shell=False)

    return callable_command


log = command('juju-log')


def run(*args):
    log('run: ' + str(args))
    return subprocess.check_output(args, shell=False)


def get_config():
    config_get = command('config-get', '--format=json')
    return json.loads(config_get())


def install_extra_repository(extra_repository):
    try:
        run('apt-add-repository', extra_repository)
        run('apt-get', 'update')
    except subprocess.CalledProcessError as e:
        log("Error adding repository %s" % extra_repository)
        log(e)
        sys.exit(-1)


def load_pickle(filepath):
    prev_config = {}
    if os.path.exists(filepath):
        with open(filepath) as fd:
            prev_config = pickle.load(fd)
    return prev_config


def save_pickle(obj, filepath):
    with open(filepath, 'w') as fd:
        pickle.dump(obj, fd)


class DictDiffer:
    """
    Calculate the difference between two dictionaries as:
    (1) items added
    (2) items removed
    (3) keys same in both but changed values
    (4) keys same in both and unchanged values
    """

    # Based on answer by hughdbrown at:
    # http://stackoverflow.com/questions/1165352

    def __init__(self, current_dict, past_dict):
        self.current_dict = current_dict
        self.past_dict = past_dict
        self.set_current = set(current_dict)
        self.set_past = set(past_dict)
        self.intersect = self.set_current.intersection(self.set_past)

    @property
    def added(self):
        return self.set_current - self.intersect

    @property
    def removed(self):
        return self.set_past - self.intersect

    @property
    def changed(self):
        return set(key for key in self.intersect
                   if self.past_dict[key] != self.current_dict[key])
    @property
    def unchanged(self):
        return set(key for key in self.intersect
                   if self.past_dict[key] == self.current_dict[key])
    @property
    def modified(self):
        return self.current_dict != self.past_dict

    @property
    def added_or_changed(self):
        return self.added.union(self.changed)

    def _changes(self, keys):
        new = {}
        old = {}
        for k in keys:
            new[k] = self.current_dict.get(k)
            old[k] = self.past_dict.get(k)
        return "%s -> %s" % (old, new)
    def __str__(self):
        if self.modified:
            s = dedent("""\
            added: %s
            removed: %s
            changed: %s
            unchanged: %s""") % (
                self._changes(self.added),
                self._changes(self.removed),
                self._changes(self.changed),
                list(self.unchanged))
        else:
            s = "no changes"
        return s


def unit_info(service_name, item_name, data=None):
    if data is None:
        data = yaml.safe_load(run('juju', 'status'))
    services = data['services'][service_name]
    units = services['units']
    item = units.items()[0][1][item_name]
    return item
